"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializeGenericFilterQuery = serializeGenericFilterQuery;
exports.deserializeGenericFilterQuery = deserializeGenericFilterQuery;
exports.convertFilterToQueryParams = convertFilterToQueryParams;
exports.parseQueryParamsToFilter = parseQueryParamsToFilter;
const operatorMap = {
    '<': 'lt',
    '>': 'gt',
    '<=': 'lte',
    '>=': 'gte',
    '=': 'eq',
    '!=': 'ne',
    'in': 'in',
    'not in': 'notin',
    'like': 'like',
    'not like': 'notlike'
};
const reverseOperatorMap = Object.entries(operatorMap)
    .reduce((acc, [key, value]) => ({ ...acc, [value]: key }), {});
const MAX_KEY_LENGTH = 100;
const MAX_VALUE_LENGTH = 1000;
const MAX_ARRAY_LENGTH = 100;
function sanitizeKey(key) {
    return key.trim();
}
function sanitizeValue(value) {
    if (!Array.isArray(value) && typeof value !== 'object' && typeof value === "string") {
        return isNaN(Number(value)) ? value : Number(value);
    }
    if (typeof value === 'string') {
        const trimmed = value.slice(0, MAX_VALUE_LENGTH).trim();
        // Attempt to convert to number if the string represents a valid number
        const num = Number(trimmed);
        return !isNaN(num) && trimmed !== '' ? num : trimmed;
    }
    if (Array.isArray(value)) {
        return value
            .slice(0, MAX_ARRAY_LENGTH)
            .map(v => {
            if (typeof v === 'string') {
                const trimmed = v.slice(0, MAX_VALUE_LENGTH).trim();
                const num = Number(trimmed);
                return !isNaN(num) && trimmed !== '' ? num : trimmed;
            }
            return v;
        });
    }
    if (typeof value === 'object' && value !== null) {
        return JSON.stringify(value).slice(0, MAX_VALUE_LENGTH);
    }
    return value;
}
function validateAndSanitizeFilter(filter) {
    if (!filter || typeof filter !== 'object') {
        throw new Error('Invalid filter: must be an object');
    }
    if (!filter.field) {
        throw new Error('Invalid keyPath: must be a non-empty string');
    }
    const sanitizedKey = sanitizeKey(filter.field);
    if (sanitizedKey.length > MAX_KEY_LENGTH) {
        throw new Error(`KeyPath exceeds maximum length of ${MAX_KEY_LENGTH} characters`);
    }
    if (!/^[a-zA-Z0-9]+(?:\.[a-zA-Z0-9]+)*$/.test(sanitizedKey)) {
        throw new Error('Invalid keyPath format: must contain only alphanumeric characters and dots');
    }
    if (!filter.operator || !operatorMap[filter.operator]) {
        throw new Error('Invalid operator');
    }
    if (filter.value === undefined || filter.value === null) {
        throw new Error('value cannot be null or undefined');
    }
    const sanitizedValue = sanitizeValue(filter.value);
    if ((filter.operator === 'in' || filter.operator === 'not in') && !Array.isArray(filter.value)) {
        throw new Error('value must be an array for "in" and "not in" operators');
    }
    return {
        field: sanitizedKey,
        operator: filter.operator,
        value: sanitizedValue
    };
}
/**
 * Serializes IGenericFilterQuery into URL query parameters.
 * @param query The generic filter query.
 * @returns A string representing the serialized query parameters.
 */
function serializeGenericFilterQuery(query) {
    const params = {};
    query.filters = query.filters.map((data) => ({
        field: data.field,
        operator: data.operator,
        value: sanitizeValue(data.value)
    }));
    if (query.filters && query.filters.length > 0) {
        params.filters = encodeURIComponent(JSON.stringify(query.filters));
    }
    if (query.pagination) {
        params.pagination = encodeURIComponent(JSON.stringify(query.pagination));
    }
    const queryString = Object.entries(params)
        .map(([key, value]) => `${key}=${value}`)
        .join('&');
    return queryString ? `?${queryString}` : '';
}
/**
 * Deserializes URL query parameters into IGenericFilterQuery.
 * @param queryString The URL query string.
 * @returns The deserialized generic filter query.
 */
function deserializeGenericFilterQuery(queryString) {
    const params = new URLSearchParams(queryString);
    const filtersParam = params.get('filters');
    const paginationParam = params.get('pagination');
    let filters = [];
    let pagination = {};
    try {
        if (filtersParam) {
            const parsedFilters = JSON.parse(decodeURIComponent(filtersParam));
            filters = parsedFilters.map(filter => validateAndSanitizeFilter(filter));
        }
    }
    catch (error) {
        console.error('Error parsing filters:', error);
    }
    try {
        if (paginationParam) {
            pagination = JSON.parse(decodeURIComponent(paginationParam));
        }
    }
    catch (error) {
        console.error('Error parsing pagination:', error);
    }
    return { filters, pagination };
}
function convertFilterToQueryParams(filter) {
    if (!filter)
        return '';
    const filters = Array.isArray(filter) ? filter : [filter];
    if (!filters.length)
        return '';
    if (filters.length > MAX_ARRAY_LENGTH) {
        throw new Error(`Too many filters: maximum allowed is ${MAX_ARRAY_LENGTH}`);
    }
    const params = filters.map(filter => {
        const sanitizedFilter = validateAndSanitizeFilter(filter);
        let processedValue = sanitizedFilter.value;
        if (Array.isArray(processedValue)) {
            processedValue = processedValue.join(',');
        }
        else if (typeof processedValue === 'object') {
            processedValue = JSON.stringify(processedValue);
        }
        const encodedValue = encodeURIComponent(String(processedValue));
        return `${sanitizedFilter.field}[${operatorMap[sanitizedFilter.operator]}]=${encodedValue}`;
    });
    return params.length ? `?${params.join('&')}` : '';
}
function parseQueryParamsToFilter(queryString) {
    if (!queryString)
        return [];
    const query = queryString.startsWith('?') ? queryString.slice(1) : queryString;
    if (!query)
        return [];
    const params = query.split('&');
    if (params.length > MAX_ARRAY_LENGTH) {
        throw new Error(`Too many parameters: maximum allowed is ${MAX_ARRAY_LENGTH}`);
    }
    return params.map(param => {
        const match = param.match(/^([^[\]]+)\[([^\]]+)]=(.+)$/);
        if (!match) {
            throw new Error(`Invalid parameter format: ${param}`);
        }
        const [, keyPath, operator, encodedValue] = match;
        const mappedOperator = reverseOperatorMap[operator];
        if (!mappedOperator) {
            throw new Error(`Invalid operator: ${operator}`);
        }
        const decodedValue = decodeURIComponent(encodedValue);
        let parsedValue = decodedValue;
        if (mappedOperator === 'in' || mappedOperator === 'not in') {
            parsedValue = decodedValue.split(',')
                .map(v => {
                const num = Number(v);
                return !isNaN(num) ? num : v.trim();
            });
        }
        else {
            const numberValue = Number(decodedValue);
            if (!isNaN(numberValue) && decodedValue !== '') {
                parsedValue = numberValue;
            }
            else if (decodedValue.startsWith('{') || decodedValue.startsWith('[')) {
                try {
                    parsedValue = JSON.parse(decodedValue);
                }
                catch {
                    parsedValue = decodedValue;
                }
            }
        }
        return validateAndSanitizeFilter({
            field: keyPath,
            operator: mappedOperator,
            value: parsedValue
        });
    });
}
//# sourceMappingURL=IFilterQuery.js.map